clear all;
close all;

global T N m b g x0 dx0 xf dxf

N=30;      % time-discretization, number of points

% problem parameters
m  = 1.4;   % car mass
b  = 0.1;   % viscous friction
g  = 9.81;  %gravitational force
T  = 12;   % time for the optimization

% Initial state
x0  =  1.0; %initial position
dx0 =  0.0; %initial velocity
% Terminal state
xf  =  4.0; %initial position
dxf =  0.0; %initial velocity

disp('%%%%%%%%%% DYNAMIC PARKING OPTIMIZATION %%%%%%%%%%');
Nu = 20;
Nx = 140;

uMin  = -1;
uMax  =  1;
xMin  = -2.0;
xMax  =  5.0;
dxMin = -4;
dxMax =  4;

xGrid  = linspace(xMin, xMax, Nx);
dxGrid = linspace(dxMin, dxMax, Nx);
uGrid  = linspace(uMin, uMax, Nu);

iFinal   = fitToGrid(xf, xGrid);
jFinal   = fitToGrid(dxf, dxGrid);
iInitial = fitToGrid(x0, xGrid);
jInitial = fitToGrid(dx0, dxGrid);

for i = 1 : Nx
    for j = 1 : Nx
        if(i==iFinal && j==jFinal)
            hf(i,j) = 0;
        else
            hf(i,j) = (i-iFinal)^2 + (j-jFinal)^2;
            %hf(i,j) = 1e10;
        end
    end
end


for i = 1 : Nx
    for j = 1 : Nx
        V(i,j,N+1) = hf(i,j);
    end
end

for k = N : -1 : 1;
    t = T * k / N;
    for i = 1 : Nx
        for j = 1 : Nx
            [tmpV, iV, jV] = computeV(uGrid, V(:,:, k+1), xGrid, dxGrid, xGrid(i), dxGrid(j));
            [V(i,j,k), iUmin] = min(1/2*uGrid.*uGrid*T/N + tmpV); %V(x,dx,t)
            uStar(i,j,k) = uGrid(iUmin);
        end
    end
    disp(k)
end

imagesc(xGrid, dxGrid, V(:,:,1))    
title(strcat('Jstar is:', num2str(Jstar), ' V is:', num2str(V(iInitial, jInitial, 1))))
colorbar

% X(:,1) = [x0; dx0];
X(:,1) = [2.7; -1];
[i, X(1,1)] = fitToGrid(X(1,1), xGrid);
[j, X(2,1)] = fitToGrid(X(2,1), dxGrid);
Jstar = 0;
for k = 1 : N
    uOpt(k) = uStar(i,j,k) ;
    Jstar = Jstar + 1/2 * uOpt(k) * uOpt(k) * T/N;
    X(:,k+1) = fwdDyn(X(:,k), uStar(i,j,k));
    [i, X(1,k+1)] = fitToGrid(X(1,k+1), xGrid);
    [j, X(2,k+1)] = fitToGrid(X(2,k+1), dxGrid);
end
Jstar = Jstar + hf(i, j);

figure
subplot(311)
plot(X(1,:))
legend('x')
subplot(312)
plot(X(2,:))
legend('dx')
subplot(313)
plot(uOpt)
legend('u')

figure;
for j = 1 : N
    xCurr = X(1,j);
    dxCurr = X(2,j);
    ezplot('atan(x)*atan(x)');
    hold on
    plot(xCurr, atan(xCurr)*atan(xCurr)+0.03, '.r', 'MarkerSize', 30);
    slopeCurr = [1, 2*atan(xCurr)/(1+xCurr^2)]/sqrt(1 + (2*atan(xCurr)/(1+xCurr^2))^2) * dxCurr;
    quiver(xCurr, atan(xCurr)*atan(xCurr)+0.03, slopeCurr(1), slopeCurr(2), 0.0,  'LineWidth', 3, 'MarkerSize', 30)

    grid
    M(j) = getframe;
    if (j ~= N)
        clf
    end
end



