clear all;
% close all;

global T N m b g x0 dx0 xf dxf

N=100;      % time-discretization, number of points

% problem parameters
m  = 1.4;   % car mass
b  = 0.1;   % viscous friction
g  = 9.81;  %gravitational force
T  = 12;   % time for the optimization

% Initial state
x0  =  1.0; %initial position
dx0 =  0.0; %initial velocity
% Terminal state
xf  =  4.0; %initial position
dxf =  0.0; %initial velocity

disp('%%%%%%%%%% PARKING OPTIMIZATION %%%%%%%%%%');

% Using a good IniGuess for the adjoint vector speeds up and helps for
% convergence... This can be obtained from the solution obtained for the
% linearized system, or from a numerical solution using GPOPS or similar
% discretized method solving NLPs.

%IniGuess = [ -1.1267   -2.1823];   %bad initial guess for T=12, m=1.4;
% b=0.1; g=9.81; x0=1.0; dx0 =0.0; xf=4.0; dxf=0.0; C = 1.5215 > C*=0.29

%IniGuess = [ -0.1031    0.1264];   %good initial guess for T=12, m=1.4;
% b=0.1; g=9.81; x0=1.0; dx0 =0.0; xf=4.0; dxf=0.0; C*=0.33

IniGuess=randn(1,2);

Result=parkingOptimization(IniGuess); % Apply Pontryagin and the shooting method

figure
subplot(311)
plot(Result.Time(:,1), Result.State(:,1))
subplot(312)
plot(Result.Time(:,1), Result.State(:,2))
subplot(313)
u = -(1/m) * Result.CoState(:,2);
plot(Result.Time(:,1), u)
uCost = (1/2) * sum(u.*u) * T/N;
title(strcat('cost is: ', ' ', num2str(uCost)))

figure;
for j = 1 : N
    xCurr = Result.State(j,1);
    ezplot('atan(x)*atan(x)');
    hold on
    plot(xCurr, atan(xCurr)*atan(xCurr)+0.03, '.r', 'MarkerSize', 30);
    grid
    M(j) = getframe;
    if (j ~= N)
        clf
    end
end
%movie(M)
%movie(M, [], N/T)

